import urllib
import numpy as np
import pandas as pd

from CHIRPS import config as cfg
from CHIRPS.structures import data_container

def usoc2(random_state=123, project_dir=None):

    vtypes = {'wave': np.object,
     'alb': np.int16,
     'alkp': np.int16,
     'alt': np.int16,
     'ast': np.int16,
     'chol': np.float16,
     'dheas': np.float16,
     'ecre': np.int16,
     'ggt': np.int16,
     'hba1c': np.int16,
     'hdl': np.float16,
     'hscrp': np.float16,
     'igfi': np.int16,
     'rtin': np.int16,
     'testo': np.float16,
     'trig': np.float16,
     'ure': np.float16,
     'hgb': np.int16,
     'cfib': np.float16,
     'uscmg': np.object,
     'uscmm': np.object,
     'cmvavc': np.object,
     'hhorig': np.object,
     'indbdub_xw': np.float16,
     'nsex': np.object,
     'confage': np.int16,
     'region': np.object,
     'medcnjd': np.object,
     'tbmed': np.object,
     'statins': np.object,
     'statina': np.object,
     'folic': np.object,
     'folpreghr': np.object,
     'diur': np.object,
     'beta': np.object,
     'aceinh': np.object,
     'calciumb': np.object,
     'obpdrug': np.object,
     'bpmedc': np.object,
     'lipid': np.object,
     'iron': np.object,
     'medtyp1': np.object,
     'medtyp2': np.object,
     'medtyp3': np.object,
     'medtyp4': np.object,
     'medtyp5': np.object,
     'medtyp6': np.object,
     'medtyp7': np.object,
     'medtyp8': np.object,
     'medtyp9': np.object,
     'medtyp10': np.object,
     'medtyp11': np.object,
     'medtyp12': np.object,
     'medtyp13': np.object,
     'medcnj': np.object,
     'resphts': np.object,
     'estht': np.float16,
     'nohtbc1': np.object,
     'nohtbc2': np.object,
     'nohtbc3': np.object,
     'nohtbc4': np.object,
     'nohtbc5': np.object,
     'nohtbc6': np.object,
     'nohtbc7': np.object,
     'nohtbc8': np.object,
     'relhite': np.object,
     'hinrel': np.object,
     'respwts': np.object,
     'bfpc': np.float16,
     'bfpcok': np.object,
     'bfpcval': np.float16,
     'nobf1': np.object,
     'nobf2': np.object,
     'nobf3': np.object,
     'resnwt': np.object,
     'nowtbc1': np.object,
     'nowtbc2': np.object,
     'nowtbc3': np.object,
     'nowtbc4': np.object,
     'nowtbc5': np.object,
     'nowtbc6': np.object,
     'nowtbc7': np.object,
     'nowtbc8': np.object,
     'nowtbc9': np.object,
     'ewtch': np.object,
     'estwt': np.float16,
     'floorc': np.object,
     'relwaitb': np.object,
     'whintro': np.object,
     'ynowh': np.object,
     'whpnabm1': np.object,
     'whpnabm2': np.object,
     'whpnabm3': np.object,
     'whpnabm4': np.object,
     'whpnabm5': np.object,
     'whpnabm6': np.object,
     'whpnabm95': np.object,
     'wjrel': np.object,
     'probwj': np.object,
     'bpconst': np.object,
     'consubx1': np.object,
     'consubx2': np.object,
     'consubx3': np.object,
     'consubx4': np.object,
     'consubx5': np.object,
     'omronno': np.int16,
     'cufsize': np.object,
     'airtemp': np.float16,
     'ynobp': np.object,
     'respbps': np.object,
     'nattbpd0': np.object,
     'nattbpd1': np.object,
     'nattbpd2': np.object,
     'nattbpd3': np.object,
     'nattbpd4': np.object,
     'nattbpd95': np.object,
     'difbpc1': np.object,
     'difbpc2': np.object,
     'difbpc3': np.object,
     'difbpc4': np.object,
     'difbpc5': np.object,
     'difbpc6': np.object,
     'difbpc95': np.object,
     'mmgswil': np.object,
     'mmgsok': np.object,
     'mmgsdval': np.int16,
     'mmgsnval': np.int16,
     'mmgsdom': np.object,
     'mmgssta': np.object,
     'mmgsd1': np.int16,
     'mmgsn1': np.int16,
     'mmgsd2': np.int16,
     'mmgsn2': np.int16,
     'mmgsd3': np.int16,
     'mmgsn3': np.int16,
     'mmgstp': np.object,
     'mmgsres': np.object,
     'mmgsprb1': np.object,
     'mmgsprb2': np.object,
     'mmgsprb3': np.object,
     'mmgsprb95': np.object,
     'lungsurg': np.object,
     'lungeye': np.object,
     'lunghrt': np.object,
     'lunghosp': np.object,
     'lungex': np.object,
     'lungtest': np.object,
     'noattlf0': np.object,
     'noattlf1': np.object,
     'noattlf2': np.object,
     'noattlf3': np.object,
     'noattlf4': np.object,
     'noattlf5': np.object,
     'noattlf95': np.object,
     'lungsmok': np.object,
     'lungsmhr': np.int16,
     'lunginhl': np.object,
     'lunginhr': np.int16,
     'htfvc': np.float16,
     'htfev': np.float16,
     'htpef': np.int16,
     'htfevfvc': np.float16,
     'fev1pred': np.float16,
     'fvcpred': np.float16,
     'fev1fvcp': np.float16,
     'htfvc_sc': np.float16,
     'htfev_sc': np.float16,
     'htpef_sc': np.float16,
     'htfevfvc_sc': np.float16,
     'fev1pred_sc': np.float16,
     'fvcpred_sc': np.float16,
     'fev1fvcp_sc': np.float16,
     'qualcdf0': np.object,
     'qualcdf1': np.object,
     'qualcdf2': np.object,
     'qualcdf3': np.object,
     'qualcdf4': np.object,
     'qualcdf5': np.object,
     'qualcdf6': np.object,
     'qualcdf7': np.object,
     'qualcdf95': np.object,
     'qualab': np.object,
     'nulllf0': np.object,
     'nulllf1': np.object,
     'nulllf2': np.object,
     'nulllf3': np.object,
     'nulllf4': np.object,
     'nulllf5': np.object,
     'nulllf6': np.object,
     'nulllf95': np.object,
     'hasurg': np.object,
     'haeysurg': np.object,
     'hastro': np.object,
     'chestinf': np.object,
     'inhaler': np.object,
     'inhalhrs': np.int16,
     'lfwill': np.object,
     'spirno': np.int16,
     'lftemp': np.float16,
     'noread': np.object,
     'nlsatlf': np.object,
     'htfvc2': np.float16,
     'ynolf': np.object,
     'lfstand': np.object,
     'lfresp': np.object,
     'problf1': np.object,
     'problf2': np.object,
     'problf3': np.object,
     'problf4': np.object,
     'problf5': np.object,
     'noattlf': np.object,
     'ncgplf': np.object,
     'ncguard': np.object,
     'clotb': np.object,
     'fit': np.object,
     'bswill': np.object,
     'refbsc1': np.object,
     'refbsc2': np.object,
     'refbsc3': np.object,
     'refbsc4': np.object,
     'refbsc5': np.object,
     'refbsc6': np.object,
     'refbsc7': np.object,
     'refbsc95': np.object,
     'constorb': np.object,
     'condna': np.object,
     'samparm': np.object,
     'samdifc1': np.object,
     'samdifc2': np.object,
     'samdifc3': np.object,
     'samdifc4': np.object,
     'samdifc5': np.object,
     'samdifc6': np.object,
     'samdifc95': np.object,
     'nobsm1': np.object,
     'nobsm2': np.object,
     'nobsm3': np.object,
     'nobsm95': np.object,
     'vpsys': np.object,
     'vphand': np.object,
     'vparm': np.object,
     'vpskin': np.object,
     'vpalco': np.object,
     'vpsam': np.object,
     'vppress1': np.object,
     'vppress2': np.object,
     'vppress3': np.object,
     'vpsens': np.object,
     'vpprob1': np.object,
     'vpprob2': np.object,
     'vpprob3': np.object,
     'vpprob95': np.object,
     'vpprob96': np.object,
     'vpcheck': np.object,
     'nseqno': np.object,
     'dateok': np.object,
     'wtpc': np.float16,
     'feet': np.object,
     'mmgspr': np.object,
     'antic': np.object,
     'bfck2': np.object,
     'nuroutc': object,
     'bsoute': np.object,
     'htok': np.object,
     'wtok': np.object,
     'bmiok': np.object,
     'htval': np.float16,
     'wtval': np.float16,
     'bmivg5': np.object,
     'lfout': object,
     'numed2': np.int16,
     'nurdayd': np.int16,
     'nurdaym': np.int16,
     'nurdayy': np.object,
     'nurdayw': np.object,
     'elig': object,
     'ethnic': object,
     'sys1': np.int16,
     'dias1': np.int16,
     'pulse1': np.int16,
     'map1': np.int16,
     'full1': np.object,
     'sys2': np.int16,
     'dias2': np.int16,
     'pulse2': np.int16,
     'map2': np.int16,
     'full2': np.object,
     'sys3': np.int16,
     'dias3': np.int16,
     'pulse3': np.int16,
     'map3': np.int16,
     'full3': np.object,
     'omsysval': np.float16,
     'omdiaval': np.float16,
     'ompulval': np.float16,
     'ommapval': np.float16,
     'omsyst': np.float16,
     'omdiast': np.float16,
     'bprespc': np.object,
     'hyper2om': np.object,
     'wstokb': np.object,
     'wstval': np.float16,
     'hhsize': np.int16,
     'ieqmoecd_dv': np.float16,
     'hhtype_dv': object,
     'jbstat': object,
     'mlstat': object,
     'sf1': np.object,
     'health': np.object,
     'marstat': object,
     'scghqa': np.object,
     'scghqb': np.object,
     'scghqc': np.object,
     'scghqd': np.object,
     'scghqe': np.object,
     'scghqf': np.object,
     'scghqg': np.object,
     'scghqh': np.object,
     'scghqi': np.object,
     'scghqj': np.object,
     'scghqk': np.object,
     'scghql': np.object,
     'scghq1_dv': np.int16,
     'scghq2_dv': np.int16,
     'hiqual_dv': np.object,
     'jbnssec8_dv': object,
     'jbnssec5_dv': np.object,
     'jbnssec3_dv': np.object,
     'jlnssec8_dv': object,
     'jlnssec5_dv': np.object,
     'jlnssec3_dv': np.object,
     'urban_dv': np.object,
     'bmival': np.float16,
     'hyper1': np.object,
     'hyper2': np.object,
     'bnf7_conhrt': np.object,
     'bnf7_antifibs': np.object,
     'bnf7_aspirin': np.object,
     'bnf7_statins': np.object,
     'bnf7_antiinflam': np.object,
     'bnf7_antiep': np.object}

    data_cont = data_container(
    data = pd.read_csv('CHIRPS' + cfg.path_sep + 'datafiles_proprietary' + cfg.path_sep + 'usoc2.csv.gz',
                        compression='gzip',
                        dtype=vtypes),
    class_col = 'mh',
    project_dir = project_dir,
    save_dir = 'usoc2',
    random_state=random_state,
    spiel = '''
    University of Essex, Institute for Social and Economic Research. (2019).
    Understanding Society: Waves 2-3 Nurse Health Assessment, 2010-2012. [data collection].
    3rd Edition. UK Data Service. SN: 7251, http://doi.org/10.5255/UKDA-SN-7251-3
    ''')
    return(data_cont)

def usoc2_samp(random_state=123, project_dir=None):

    vtypes = {'wave': np.object,
     'alb': np.int16,
     'alkp': np.int16,
     'alt': np.int16,
     'ast': np.int16,
     'chol': np.float16,
     'dheas': np.float16,
     'ecre': np.int16,
     'ggt': np.int16,
     'hba1c': np.int16,
     'hdl': np.float16,
     'hscrp': np.float16,
     'igfi': np.int16,
     'rtin': np.int16,
     'testo': np.float16,
     'trig': np.float16,
     'ure': np.float16,
     'hgb': np.int16,
     'cfib': np.float16,
     'uscmg': np.object,
     'uscmm': np.object,
     'cmvavc': np.object,
     'hhorig': np.object,
     'indbdub_xw': np.float16,
     'nsex': np.object,
     'confage': np.int16,
     'region': np.object,
     'medcnjd': np.object,
     'tbmed': np.object,
     'statins': np.object,
     'statina': np.object,
     'folic': np.object,
     'folpreghr': np.object,
     'diur': np.object,
     'beta': np.object,
     'aceinh': np.object,
     'calciumb': np.object,
     'obpdrug': np.object,
     'bpmedc': np.object,
     'lipid': np.object,
     'iron': np.object,
     'medtyp1': np.object,
     'medtyp2': np.object,
     'medtyp3': np.object,
     'medtyp4': np.object,
     'medtyp5': np.object,
     'medtyp6': np.object,
     'medtyp7': np.object,
     'medtyp8': np.object,
     'medtyp9': np.object,
     'medtyp10': np.object,
     'medtyp11': np.object,
     'medtyp12': np.object,
     'medtyp13': np.object,
     'medcnj': np.object,
     'resphts': np.object,
     'estht': np.float16,
     'nohtbc1': np.object,
     'nohtbc2': np.object,
     'nohtbc3': np.object,
     'nohtbc4': np.object,
     'nohtbc5': np.object,
     'nohtbc6': np.object,
     'nohtbc7': np.object,
     'nohtbc8': np.object,
     'relhite': np.object,
     'hinrel': np.object,
     'respwts': np.object,
     'bfpc': np.float16,
     'bfpcok': np.object,
     'bfpcval': np.float16,
     'nobf1': np.object,
     'nobf2': np.object,
     'nobf3': np.object,
     'resnwt': np.object,
     'nowtbc1': np.object,
     'nowtbc2': np.object,
     'nowtbc3': np.object,
     'nowtbc4': np.object,
     'nowtbc5': np.object,
     'nowtbc6': np.object,
     'nowtbc7': np.object,
     'nowtbc8': np.object,
     'nowtbc9': np.object,
     'ewtch': np.object,
     'estwt': np.float16,
     'floorc': np.object,
     'relwaitb': np.object,
     'whintro': np.object,
     'ynowh': np.object,
     'whpnabm1': np.object,
     'whpnabm2': np.object,
     'whpnabm3': np.object,
     'whpnabm4': np.object,
     'whpnabm5': np.object,
     'whpnabm6': np.object,
     'whpnabm95': np.object,
     'wjrel': np.object,
     'probwj': np.object,
     'bpconst': np.object,
     'consubx1': np.object,
     'consubx2': np.object,
     'consubx3': np.object,
     'consubx4': np.object,
     'consubx5': np.object,
     'omronno': np.int16,
     'cufsize': np.object,
     'airtemp': np.float16,
     'ynobp': np.object,
     'respbps': np.object,
     'nattbpd0': np.object,
     'nattbpd1': np.object,
     'nattbpd2': np.object,
     'nattbpd3': np.object,
     'nattbpd4': np.object,
     'nattbpd95': np.object,
     'difbpc1': np.object,
     'difbpc2': np.object,
     'difbpc3': np.object,
     'difbpc4': np.object,
     'difbpc5': np.object,
     'difbpc6': np.object,
     'difbpc95': np.object,
     'mmgswil': np.object,
     'mmgsok': np.object,
     'mmgsdval': np.int16,
     'mmgsnval': np.int16,
     'mmgsdom': np.object,
     'mmgssta': np.object,
     'mmgsd1': np.int16,
     'mmgsn1': np.int16,
     'mmgsd2': np.int16,
     'mmgsn2': np.int16,
     'mmgsd3': np.int16,
     'mmgsn3': np.int16,
     'mmgstp': np.object,
     'mmgsres': np.object,
     'mmgsprb1': np.object,
     'mmgsprb2': np.object,
     'mmgsprb3': np.object,
     'mmgsprb95': np.object,
     'lungsurg': np.object,
     'lungeye': np.object,
     'lunghrt': np.object,
     'lunghosp': np.object,
     'lungex': np.object,
     'lungtest': np.object,
     'noattlf0': np.object,
     'noattlf1': np.object,
     'noattlf2': np.object,
     'noattlf3': np.object,
     'noattlf4': np.object,
     'noattlf5': np.object,
     'noattlf95': np.object,
     'lungsmok': np.object,
     'lungsmhr': np.int16,
     'lunginhl': np.object,
     'lunginhr': np.int16,
     'htfvc': np.float16,
     'htfev': np.float16,
     'htpef': np.int16,
     'htfevfvc': np.float16,
     'fev1pred': np.float16,
     'fvcpred': np.float16,
     'fev1fvcp': np.float16,
     'htfvc_sc': np.float16,
     'htfev_sc': np.float16,
     'htpef_sc': np.float16,
     'htfevfvc_sc': np.float16,
     'fev1pred_sc': np.float16,
     'fvcpred_sc': np.float16,
     'fev1fvcp_sc': np.float16,
     'qualcdf0': np.object,
     'qualcdf1': np.object,
     'qualcdf2': np.object,
     'qualcdf3': np.object,
     'qualcdf4': np.object,
     'qualcdf5': np.object,
     'qualcdf6': np.object,
     'qualcdf7': np.object,
     'qualcdf95': np.object,
     'qualab': np.object,
     'nulllf0': np.object,
     'nulllf1': np.object,
     'nulllf2': np.object,
     'nulllf3': np.object,
     'nulllf4': np.object,
     'nulllf5': np.object,
     'nulllf6': np.object,
     'nulllf95': np.object,
     'hasurg': np.object,
     'haeysurg': np.object,
     'hastro': np.object,
     'chestinf': np.object,
     'inhaler': np.object,
     'inhalhrs': np.int16,
     'lfwill': np.object,
     'spirno': np.int16,
     'lftemp': np.float16,
     'noread': np.object,
     'nlsatlf': np.object,
     'htfvc2': np.float16,
     'ynolf': np.object,
     'lfstand': np.object,
     'lfresp': np.object,
     'problf1': np.object,
     'problf2': np.object,
     'problf3': np.object,
     'problf4': np.object,
     'problf5': np.object,
     'noattlf': np.object,
     'ncgplf': np.object,
     'ncguard': np.object,
     'clotb': np.object,
     'fit': np.object,
     'bswill': np.object,
     'refbsc1': np.object,
     'refbsc2': np.object,
     'refbsc3': np.object,
     'refbsc4': np.object,
     'refbsc5': np.object,
     'refbsc6': np.object,
     'refbsc7': np.object,
     'refbsc95': np.object,
     'constorb': np.object,
     'condna': np.object,
     'samparm': np.object,
     'samdifc1': np.object,
     'samdifc2': np.object,
     'samdifc3': np.object,
     'samdifc4': np.object,
     'samdifc5': np.object,
     'samdifc6': np.object,
     'samdifc95': np.object,
     'nobsm1': np.object,
     'nobsm2': np.object,
     'nobsm3': np.object,
     'nobsm95': np.object,
     'vpsys': np.object,
     'vphand': np.object,
     'vparm': np.object,
     'vpskin': np.object,
     'vpalco': np.object,
     'vpsam': np.object,
     'vppress1': np.object,
     'vppress2': np.object,
     'vppress3': np.object,
     'vpsens': np.object,
     'vpprob1': np.object,
     'vpprob2': np.object,
     'vpprob3': np.object,
     'vpprob95': np.object,
     'vpprob96': np.object,
     'vpcheck': np.object,
     'nseqno': np.object,
     'dateok': np.object,
     'wtpc': np.float16,
     'feet': np.object,
     'mmgspr': np.object,
     'antic': np.object,
     'bfck2': np.object,
     'nuroutc': object,
     'bsoute': np.object,
     'htok': np.object,
     'wtok': np.object,
     'bmiok': np.object,
     'htval': np.float16,
     'wtval': np.float16,
     'bmivg5': np.object,
     'lfout': object,
     'numed2': np.int16,
     'nurdayd': np.int16,
     'nurdaym': np.int16,
     'nurdayy': np.object,
     'nurdayw': np.object,
     'elig': object,
     'ethnic': object,
     'sys1': np.int16,
     'dias1': np.int16,
     'pulse1': np.int16,
     'map1': np.int16,
     'full1': np.object,
     'sys2': np.int16,
     'dias2': np.int16,
     'pulse2': np.int16,
     'map2': np.int16,
     'full2': np.object,
     'sys3': np.int16,
     'dias3': np.int16,
     'pulse3': np.int16,
     'map3': np.int16,
     'full3': np.object,
     'omsysval': np.float16,
     'omdiaval': np.float16,
     'ompulval': np.float16,
     'ommapval': np.float16,
     'omsyst': np.float16,
     'omdiast': np.float16,
     'bprespc': np.object,
     'hyper2om': np.object,
     'wstokb': np.object,
     'wstval': np.float16,
     'hhsize': np.int16,
     'ieqmoecd_dv': np.float16,
     'hhtype_dv': object,
     'jbstat': object,
     'mlstat': object,
     'sf1': np.object,
     'health': np.object,
     'marstat': object,
     'scghqa': np.object,
     'scghqb': np.object,
     'scghqc': np.object,
     'scghqd': np.object,
     'scghqe': np.object,
     'scghqf': np.object,
     'scghqg': np.object,
     'scghqh': np.object,
     'scghqi': np.object,
     'scghqj': np.object,
     'scghqk': np.object,
     'scghql': np.object,
     'scghq1_dv': np.int16,
     'scghq2_dv': np.int16,
     'hiqual_dv': np.object,
     'jbnssec8_dv': object,
     'jbnssec5_dv': np.object,
     'jbnssec3_dv': np.object,
     'jlnssec8_dv': object,
     'jlnssec5_dv': np.object,
     'jlnssec3_dv': np.object,
     'urban_dv': np.object,
     'bmival': np.float16,
     'hyper1': np.object,
     'hyper2': np.object,
     'bnf7_conhrt': np.object,
     'bnf7_antifibs': np.object,
     'bnf7_aspirin': np.object,
     'bnf7_statins': np.object,
     'bnf7_antiinflam': np.object,
     'bnf7_antiep': np.object}

    data_cont = data_container(
    data = pd.read_csv('CHIRPS' + cfg.path_sep + 'datafiles_proprietary' + cfg.path_sep + 'usoc2_samp.csv.gz',
                        compression='gzip',
                        dtype=vtypes),
    class_col = 'mh',
    project_dir = project_dir,
    save_dir = 'usoc2_samp',
    random_state=random_state,
    spiel = '''
    University of Essex, Institute for Social and Economic Research. (2019).
    Understanding Society: Waves 2-3 Nurse Health Assessment, 2010-2012. [data collection].
    3rd Edition. UK Data Service. SN: 7251, http://doi.org/10.5255/UKDA-SN-7251-3
    ''')
    return(data_cont)
